@@ -1,25 +1,24 @@
 """Spectral biclustering algorithms."""
-# Authors : Kemal Eren
-# License: BSD 3 clause
+
+# Authors: The scikit-learn developers
+# SPDX-License-Identifier: BSD-3-Clause
 
 from abc import ABCMeta, abstractmethod
+from numbers import Integral
 
 import numpy as np
-
 from scipy.linalg import norm
 from scipy.sparse import dia_matrix, issparse
 from scipy.sparse.linalg import eigsh, svds
 
-from . import KMeans, MiniBatchKMeans
-from ..base import BaseEstimator, BiclusterMixin
-from ..utils import check_random_state
+from sklearn.base import BaseEstimator, BiclusterMixin, _fit_context
+from sklearn.cluster._kmeans import KMeans, MiniBatchKMeans
+from sklearn.utils import check_random_state, check_scalar
+from sklearn.utils._param_validation import Interval, StrOptions
+from sklearn.utils.extmath import _randomized_svd, make_nonnegative, safe_sparse_dot
+from sklearn.utils.validation import assert_all_finite, validate_data
 
-from ..utils.extmath import make_nonnegative, randomized_svd, safe_sparse_dot
-
-from ..utils.validation import assert_all_finite
-
-
-__all__ = ["SpectralCoclustering", "SpectralBiclustering"]
+__all__ = ["SpectralBiclustering", "SpectralCoclustering"]
 
 
 def _scale_normalize(X):
@@ -27,7 +26,6 @@ def _scale_normalize(X):
 
     Returns the normalized matrix and the row and column scaling
     factors.
-
     """
     X = make_nonnegative(X)
     row_diag = np.asarray(1.0 / np.sqrt(X.sum(axis=1))).squeeze()
@@ -38,7 +36,7 @@ def _scale_normalize(X):
         n_rows, n_cols = X.shape
         r = dia_matrix((row_diag, [0]), shape=(n_rows, n_rows))
         c = dia_matrix((col_diag, [0]), shape=(n_cols, n_cols))
-        an = r * X * c
+        an = r @ X @ c
     else:
         an = row_diag[:, np.newaxis] * X * col_diag
     return an, row_diag, col_diag
@@ -48,7 +46,6 @@ def _bistochastic_normalize(X, max_iter=1000, tol=1e-5):
     """Normalize rows and columns of ``X`` simultaneously so that all
     rows sum to one constant and all columns sum to a different
     constant.
-
     """
     # According to paper, this can also be done more efficiently with
     # deviation reduction and balancing algorithms.
@@ -85,6 +82,15 @@ def _log_normalize(X):
 class BaseSpectral(BiclusterMixin, BaseEstimator, metaclass=ABCMeta):
     """Base class for spectral biclustering."""
 
+    _parameter_constraints: dict = {
+        "svd_method": [StrOptions({"randomized", "arpack"})],
+        "n_svd_vecs": [Interval(Integral, 0, None, closed="left"), None],
+        "mini_batch": ["boolean"],
+        "init": [StrOptions({"k-means++", "random"}), np.ndarray],
+        "n_init": [Interval(Integral, 1, None, closed="left")],
+        "random_state": ["random_state"],
+    }
+
     @abstractmethod
     def __init__(
         self,
@@ -104,15 +110,11 @@ class BaseSpectral(BiclusterMixin, BaseEstimator, metaclass=ABCMeta):
         self.n_init = n_init
         self.random_state = random_state
 
-    def _check_parameters(self):
-        legal_svd_methods = ("randomized", "arpack")
-        if self.svd_method not in legal_svd_methods:
-            raise ValueError(
-                "Unknown SVD method: '{0}'. svd_method must be one of {1}.".format(
-                    self.svd_method, legal_svd_methods
-                )
-            )
+    @abstractmethod
+    def _check_parameters(self, n_samples):
+        """Validate parameters depending on the input data."""
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y=None):
         """Create a biclustering for X.
 
@@ -129,21 +131,20 @@ class BaseSpectral(BiclusterMixin, BaseEstimator, metaclass=ABCMeta):
         self : object
             SpectralBiclustering instance.
         """
-        X = self._validate_data(X, accept_sparse="csr", dtype=np.float64)
-        self._check_parameters()
+        X = validate_data(self, X, accept_sparse="csr", dtype=np.float64)
+        self._check_parameters(X.shape[0])
         self._fit(X)
         return self
 
     def _svd(self, array, n_components, n_discard):
         """Returns first `n_components` left and right singular
         vectors u and v, discarding the first `n_discard`.
-
         """
         if self.svd_method == "randomized":
             kwargs = {}
             if self.n_svd_vecs is not None:
                 kwargs["n_oversamples"] = self.n_svd_vecs
-            u, _, vt = randomized_svd(
+            u, _, vt = _randomized_svd(
                 array, n_components, random_state=self.random_state, **kwargs
             )
 
@@ -192,22 +193,14 @@ class BaseSpectral(BiclusterMixin, BaseEstimator, metaclass=ABCMeta):
         labels = model.labels_
         return centroid, labels
 
-    def _more_tags(self):
-        return {
-            "_xfail_checks": {
-                "check_estimators_dtypes": "raises nan error",
-                "check_fit2d_1sample": "_scale_normalize fails",
-                "check_fit2d_1feature": "raises apply_along_axis error",
-                "check_estimator_sparse_data": "does not fail gracefully",
-                "check_methods_subset_invariance": "empty array passed inside",
-                "check_dont_overwrite_parameters": "empty array passed inside",
-                "check_fit2d_predict1d": "empty array passed inside",
-            }
-        }
+    def __sklearn_tags__(self):
+        tags = super().__sklearn_tags__()
+        tags.input_tags.sparse = True
+        return tags
 
 
 class SpectralCoclustering(BaseSpectral):
-    """Spectral Co-Clustering algorithm (Dhillon, 2001).
+    """Spectral Co-Clustering algorithm (Dhillon, 2001) [1]_.
 
     Clusters rows and columns of an array `X` to solve the relaxed
     normalized cut of the bipartite graph created from `X` as follows:
@@ -243,7 +236,7 @@ class SpectralCoclustering(BaseSpectral):
         Whether to use mini-batch k-means, which is faster but may get
         different results.
 
-    init : {'k-means++', 'random', or ndarray of shape \
+    init : {'k-means++', 'random'}, or ndarray of shape \
             (n_clusters, n_features), default='k-means++'
         Method for initialization of k-means algorithm; defaults to
         'k-means++'.
@@ -290,6 +283,17 @@ class SpectralCoclustering(BaseSpectral):
 
         .. versionadded:: 1.0
 
+    See Also
+    --------
+    SpectralBiclustering : Partitions rows and columns under the assumption
+        that the data has an underlying checkerboard structure.
+
+    References
+    ----------
+    .. [1] :doi:`Dhillon, Inderjit S, 2001. Co-clustering documents and words using
+           bipartite spectral graph partitioning.
+           <10.1145/502512.502550>`
+
     Examples
     --------
     >>> from sklearn.cluster import SpectralCoclustering
@@ -304,15 +308,15 @@ class SpectralCoclustering(BaseSpectral):
     >>> clustering
     SpectralCoclustering(n_clusters=2, random_state=0)
 
-    References
-    ----------
-
-    * Dhillon, Inderjit S, 2001. `Co-clustering documents and words using
-      bipartite spectral graph partitioning
-      <http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.140.3011>`__.
-
+    For a more detailed example, see the following:
+    :ref:`sphx_glr_auto_examples_bicluster_plot_spectral_coclustering.py`.
     """
 
+    _parameter_constraints: dict = {
+        **BaseSpectral._parameter_constraints,
+        "n_clusters": [Interval(Integral, 1, None, closed="left")],
+    }
+
     def __init__(
         self,
         n_clusters=3,
@@ -328,6 +332,13 @@ class SpectralCoclustering(BaseSpectral):
             n_clusters, svd_method, n_svd_vecs, mini_batch, init, n_init, random_state
         )
 
+    def _check_parameters(self, n_samples):
+        if self.n_clusters > n_samples:
+            raise ValueError(
+                f"n_clusters should be <= n_samples={n_samples}. Got"
+                f" {self.n_clusters} instead."
+            )
+
     def _fit(self, X):
         normalized_data, row_diag, col_diag = _scale_normalize(X)
         n_sv = 1 + int(np.ceil(np.log2(self.n_clusters)))
@@ -347,7 +358,7 @@ class SpectralCoclustering(BaseSpectral):
 
 
 class SpectralBiclustering(BaseSpectral):
-    """Spectral biclustering (Kluger, 2003).
+    """Spectral biclustering (Kluger, 2003) [1]_.
 
     Partitions rows and columns under the assumption that the data has
     an underlying checkerboard structure. For instance, if there are
@@ -372,7 +383,7 @@ class SpectralBiclustering(BaseSpectral):
         default is 'bistochastic'.
 
         .. warning::
-           if `method='log'`, the data must be sparse.
+           if `method='log'`, the data must not be sparse.
 
     n_components : int, default=6
         Number of singular vectors to check.
@@ -398,7 +409,7 @@ class SpectralBiclustering(BaseSpectral):
         Whether to use mini-batch k-means, which is faster but may get
         different results.
 
-    init : {'k-means++', 'random'} or ndarray of (n_clusters, n_features), \
+    init : {'k-means++', 'random'} or ndarray of shape (n_clusters, n_features), \
             default='k-means++'
         Method for initialization of k-means algorithm; defaults to
         'k-means++'.
@@ -447,14 +458,15 @@ class SpectralBiclustering(BaseSpectral):
 
     See Also
     --------
-    SpectralCoclustering : Spectral Co-Clustering algorithm (Dhillon, 2001).
+    SpectralCoclustering : Clusters rows and columns of an array `X` to solve the
+        relaxed normalized cut of the bipartite graph created from `X`.
 
     References
     ----------
 
-    * Kluger, Yuval, et. al., 2003. `Spectral biclustering of microarray
-      data: coclustering genes and conditions
-      <http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.135.1608>`__.
+    .. [1] :doi:`Kluger, Yuval, et. al., 2003. Spectral biclustering of microarray
+           data: coclustering genes and conditions.
+           <10.1101/gr.648603>`
 
     Examples
     --------
@@ -466,11 +478,22 @@ class SpectralBiclustering(BaseSpectral):
     >>> clustering.row_labels_
     array([1, 1, 1, 0, 0, 0], dtype=int32)
     >>> clustering.column_labels_
-    array([0, 1], dtype=int32)
+    array([1, 0], dtype=int32)
     >>> clustering
     SpectralBiclustering(n_clusters=2, random_state=0)
+
+    For a more detailed example, see
+    :ref:`sphx_glr_auto_examples_bicluster_plot_spectral_biclustering.py`
     """
 
+    _parameter_constraints: dict = {
+        **BaseSpectral._parameter_constraints,
+        "n_clusters": [Interval(Integral, 1, None, closed="left"), tuple],
+        "method": [StrOptions({"bistochastic", "scale", "log"})],
+        "n_components": [Interval(Integral, 1, None, closed="left")],
+        "n_best": [Interval(Integral, 1, None, closed="left")],
+    }
+
     def __init__(
         self,
         n_clusters=3,
@@ -492,45 +515,43 @@ class SpectralBiclustering(BaseSpectral):
         self.n_components = n_components
         self.n_best = n_best
 
-    def _check_parameters(self):
-        super()._check_parameters()
-        legal_methods = ("bistochastic", "scale", "log")
-        if self.method not in legal_methods:
-            raise ValueError(
-                "Unknown method: '{0}'. method must be one of {1}.".format(
-                    self.method, legal_methods
+    def _check_parameters(self, n_samples):
+        if isinstance(self.n_clusters, Integral):
+            if self.n_clusters > n_samples:
+                raise ValueError(
+                    f"n_clusters should be <= n_samples={n_samples}. Got"
+                    f" {self.n_clusters} instead."
                 )
-            )
-        try:
-            int(self.n_clusters)
-        except TypeError:
+        else:  # tuple
             try:
-                r, c = self.n_clusters
-                int(r)
-                int(c)
+                n_row_clusters, n_column_clusters = self.n_clusters
+                check_scalar(
+                    n_row_clusters,
+                    "n_row_clusters",
+                    target_type=Integral,
+                    min_val=1,
+                    max_val=n_samples,
+                )
+                check_scalar(
+                    n_column_clusters,
+                    "n_column_clusters",
+                    target_type=Integral,
+                    min_val=1,
+                    max_val=n_samples,
+                )
             except (ValueError, TypeError) as e:
                 raise ValueError(
                     "Incorrect parameter n_clusters has value:"
-                    " {}. It should either be a single integer"
+                    f" {self.n_clusters}. It should either be a single integer"
                     " or an iterable with two integers:"
                     " (n_row_clusters, n_column_clusters)"
+                    " And the values are should be in the"
+                    " range: (1, n_samples)"
                 ) from e
-        if self.n_components < 1:
-            raise ValueError(
-                "Parameter n_components must be greater than 0,"
-                " but its value is {}".format(self.n_components)
-            )
-        if self.n_best < 1:
-            raise ValueError(
-                "Parameter n_best must be greater than 0, but its value is {}".format(
-                    self.n_best
-                )
-            )
+
         if self.n_best > self.n_components:
             raise ValueError(
-                "n_best cannot be larger than n_components, but {} >  {}".format(
-                    self.n_best, self.n_components
-                )
+                f"n_best={self.n_best} must be <= n_components={self.n_components}."
             )
 
     def _fit(self, X):
